from typing import Union

from ..base import ls
from ..constants import PYSLK_LOGGER, PYSLK_WILDCARDS
from ..core import _group_retrieve
from ..raw import retrieve_raw

__all__ = [
    "retrieve",
]


def retrieve(
    resource: Union[str, list, int],
    dest_dir: Union[str, None] = None,
    recursive: bool = False,
    group: Union[bool, None] = None,
    delayed: bool = False,
    **kwargs,
) -> Union[str, list]:
    """Retrieve data from tape archive using slk retrieve

    If group is True or resource is a list of files, the retrieve calls will
    be grouped by tape id to optimize retrieve calls.
    This functions aims at implementing the recommendations of DKRZ for
    `speeding up retrievals <https://docs.dkrz.de/doc/datastorage/hsm/retrievals.html#speed-up-your-retrievals>`_.

    :param resource:
        Path, pattern, SearchID, file or list of files that should be retrieved.
    :type resource: ``str``, ``list`` or ``int``
    :param dest_dir:
        Destination directory for retrieval. Retrieves to current directory by default.
    :type dest_dir: ``str``
    :param recursive:
        Retrieve recursively.
    :type recursive: ``bool``
    :param group:
        Group files by tape id and make one retrieve call per group. If group is None,
        retrieve calls will only be grouped if resource is a list of files.
        To totally avoid grouping, set group to False.
    :type group: ``bool``
    :param delayed:
        Delay retrieve calls using dask delayed.
    :type delayed: ``bool``
    :return: StdOut from slk calls. If delayed is True, a list of ``dask.delayed.Delayed``
        objects per tape_id is returned for later (maybe parallel) execution.
    :rtype: ``str`` or ``list``

    .. seealso::
        * :py:meth:`~pyslk.retrieve_raw`

    """

    if dest_dir is None:
        dest_dir = "."

    if group is True or (isinstance(resource, (list, int)) and group is None):
        return _group_retrieve(resource, dest_dir, recursive, delayed, **kwargs)

    if delayed is True:
        from dask import delayed as delay
    else:

        def delay(x):
            return x

    if isinstance(resource, str) and PYSLK_WILDCARDS in resource:
        # create file list, slk retrieve does not support this
        resource = list(ls(resource).filename)

    PYSLK_LOGGER.debug(f"resource: {resource}")

    if isinstance(resource, list):
        return [
            delay(retrieve_raw)(r, dest_dir, recursive=recursive, **kwargs)
            for r in resource
        ]

    return delay(retrieve_raw)(resource, dest_dir, recursive=recursive, **kwargs)


# def check_retrieve_permissions_dst(
#     path_or_id: Union[Path, str, int],
#     dest_dir: Union[str, Path],
#     recursive: bool = False,
#     duplicate: bool = False,
#     preserve_path: bool = False,
#     skip_if_exists: bool = False,
# ) -> set:
#     """Retrieve files via search id or GNS path.
#
#     Overwrite files if they already exists. Prevent this by 'duplicate'
#     or 'skip_if_exists'.
#
#     :param path_or_id: search id or gns path of resources to retrieve
#     :type path_or_id: ``str`` or ``int``
#     :param dest_dir: destination directory for retrieval
#     :type dest_dir: ``str``
#     :param recursive: use the -R flag to retrieve recursively, Default: False
#     :type recursive: ``bool``
#     :param duplicate: create a duplicate file if file exists
#     :type duplicate: ``bool``
#     :param preserve_path: preserve namespace in destination
#     :type preserve_path: ``bool``
#     :param skip_if_exists: Skip if file exists
#     :type skip_if_exists: ``bool``
#     :returns: stdout of the slk call
#     :rtype: ``str``
#     """
#     output = {"allowed": dict(), "not_allowed": dict(), "root_problem": dict()}
#     files = construct_target_file_path_local(path_or_id, dest_dir, recursive, preserve_path)
